@@ -1,10 +1,9 @@
-
-import logging
 import os
 import shutil
 import tempfile
 import atexit
 
+import torch
 
 class ConstsMetaClass(type):
     @property
@@ -23,6 +22,6 @@ class Consts(metaclass=ConstsMetaClass):
     Name = "mcquic"
     # lazy load
     # TempDir = "/tmp/mcquic/"
-    Eps = 1e-8
+    Eps = 1e-6
     CDot = "·"
-    TimeOut = 30
+    TimeOut = 15
